@@ -38,6 +38,7 @@
 #include <sys/ioctl.h>
 
 #include <libinput.h>
+#include <libevdev/libevdev.h>
 #include "config.h"
 #include "shared.h"
 #include "consolation.h"
@@ -50,11 +51,14 @@ enum mouse_reporting_mode mouse_reporting = MOUSE_REPORTING_OFF;
 
 static struct tools_options options;
 static volatile unsigned int stop = 0;
-static enum tools_backend backend = BACKEND_UDEV;
-static const char *seat_or_device = "seat0";
+static enum tools_backend backend = BACKEND_NONE;
+#define MAX_DEVICES 64
+static const char *seat_or_devices[MAX_DEVICES+1] = {"seat0"};
+static size_t ndevices = 0;
 static bool grab = false;
 static bool verbose = false;
 static const char *word_chars = NULL;
+static int action_map[256];
 
 static void
 handle_motion_event(struct libinput_event *ev)
@@ -81,6 +85,8 @@ handle_pointer_button_event(struct libinput_event *ev)
   enum libinput_button_state state;
   int button;
   button = libinput_event_pointer_get_button(p);
+  if (button >= 0x100 && button < 0x200)
+    button = action_map[button - 0x100] + 0x100;
   state = libinput_event_pointer_get_button_state(p);
   switch(button)
   {
@@ -217,13 +223,13 @@ mainloop(struct libinput *li)
     handle_events(li);
 }
 
-void
+int
 usage(void)
 {
   printf("Usage: %s [options] [--udev [<seat>]|--device /dev/input/event0]\n"
          "--udev <seat>.... Use udev device discovery (default).\n"
          "                  Specifying a seat ID is optional.\n"
-         "--device /path/to/device .... open the given device only\n"
+         "--device /path/to/device .... open the given device(s) only\n"
          "--apply-to=\"pattern\"....  Configuration options are only applied where the device name matches the pattern\n"
          "--disable-sendevents=\"pattern\".... Set the send-events option to disabled for the devices matching patterns\n"
          "\n"
@@ -241,7 +247,11 @@ usage(void)
          "--enable-middlebutton\n"
          "--disable-middlebutton.... enable/disable middle button emulation\n"
          "--enable-dwt\n"
-         "--disable-dwt..... enable/disable disable-while-typing\n"
+         "--disable-dwt.... enable/disable disable-while-typing\n"
+#ifdef HAVE_LIBINPUT_1_15
+         "--enable-scroll-button-lock\n"
+         "--disable-scroll-button-lock.... enable/disable the scroll button lock\n"
+#endif
          "--set-click-method=[none|clickfinger|buttonareas] .... set the desired click method\n"
          "--set-scroll-method=[none|twofinger|edge|button] ... set the desired scroll method\n"
          "--set-scroll-button=BTN_MIDDLE ... set the button to the given button code\n"
@@ -256,11 +266,13 @@ usage(void)
          "--word-chars=<string>.... List of characters that make up words.\n"
          "                          Ranges (a-z, A-Z, 0-9 etc.) are allowed.\n"
          "--grab .......... Exclusively grab all opened devices.\n"
+         "--map button=action set button to action.\n"
          "--no-daemon...... Do not detach and run in the background.\n"
          "--verbose ....... Print debugging output.\n"
          "--version ....... Print version information.\n"
          "--help .......... Print this help.\n",
          program_invocation_short_name);
+  return 1;
 }
 
 static void
@@ -269,11 +281,31 @@ version(void)
   printf("%s %s\n", PACKAGE_NAME, PACKAGE_VERSION);
 }
 
+static void
+init_map(void)
+{
+  int i;
+  for (i=0; i<256; i++)
+    action_map[i] = i;
+}
+
+static int
+decode_action(char *act)
+{
+  if (!strcmp(act,"select") || !strcmp(act,"SELECT"))
+    return BTN_LEFT;
+  if (!strcmp(act,"paste")   || !strcmp(act,"PASTE"))
+    return BTN_MIDDLE;
+  if (!strcmp(act,"endselect")  || !strcmp(act,"ENDSELECT"))
+    return BTN_RIGHT;
+  return libevdev_event_code_from_name(EV_KEY, act);
+}
+
 static int
 parse_args(int argc, char **argv)
 {
   tools_init_options(&options);
-
+  init_map();
   while (1) {
     int c;
     int option_index = 0;
@@ -281,6 +313,7 @@ parse_args(int argc, char **argv)
       OPT_DEVICE = 1,
       OPT_UDEV,
       OPT_GRAB,
+      OPT_MAP,
       OPT_NO_DAEMON,
       OPT_HELP,
       OPT_VERBOSE,
@@ -293,6 +326,7 @@ parse_args(int argc, char **argv)
       { "device",                    required_argument, 0, OPT_DEVICE },
       { "udev",                      required_argument, 0, OPT_UDEV },
       { "grab",                      no_argument,       0, OPT_GRAB },
+      { "map",                       required_argument, 0, OPT_MAP },
       { "no-daemon",                 no_argument,       0, OPT_NO_DAEMON },
       { "verbose",                   no_argument,       0, OPT_VERBOSE },
       { "version",                   no_argument,       0, OPT_VERSION },
@@ -317,12 +351,22 @@ parse_args(int argc, char **argv)
       exit(0);
       break;
     case OPT_DEVICE:
+      if (backend == BACKEND_UDEV)
+        return usage();
       backend = BACKEND_DEVICE;
-      seat_or_device = optarg;
+      if (ndevices >= MAX_DEVICES)
+      {
+        fprintf(stderr, "Too many --device options\n");
+        return 1;
+      }
+      seat_or_devices[ndevices++] = optarg;
       break;
     case OPT_UDEV:
+      if (backend == BACKEND_DEVICE)
+        return usage();
       backend = BACKEND_UDEV;
-      seat_or_device = optarg;
+      seat_or_devices[0] = optarg;
+      ndevices = 1;
       break;
     case OPT_GRAB:
       grab = true;
@@ -336,6 +380,23 @@ parse_args(int argc, char **argv)
     case OPT_WORD_CHARS:
       word_chars = optarg;
       break;
+    case OPT_MAP:
+    {
+      int button, action;
+      char *btn = strdup(optarg), *act;
+      char *eq = strchr(btn,'=');
+      if (!eq)
+        return usage();
+      *eq = 0; act = eq+1;
+      button = libevdev_event_code_from_name(EV_KEY, btn);
+      action = decode_action(act);
+      free(btn);
+      if (button == -1 || action == -1)
+        return usage();
+      if (button >= 0x100 && button < 0x200)
+        action_map[button - 0x100] = action - 0x100;
+    }
+      break;
     default:
       if (tools_parse_option(c, optarg, &options) != 0) {
         usage();
@@ -348,6 +409,12 @@ parse_args(int argc, char **argv)
     usage();
     return 1;
   }
+  if (backend == BACKEND_NONE)
+  {
+    backend = BACKEND_UDEV;
+    ndevices = 1;
+  }
+  seat_or_devices[ndevices] = NULL;
   return 0;
 }
 
@@ -376,7 +443,7 @@ event_main(void)
   }
 
   set_lut(word_chars);
-  li = tools_open_backend(backend, seat_or_device, verbose, &grab);
+  li = tools_open_backend(backend, seat_or_devices, verbose, &grab);
   if (!li)
     return 1;
 
